==> sapdb-source-7.3.00.48.tgz: V73_00_48/SAPDB_ORG/sys/src/eo/geo57.h <==
/*!**************************************************************************

  module      : geo57.h

  -------------------------------------------------------------------------

  responsible : RaymondR

  special area: Memory management functions.

  description : Memory management functions.


  last changed: 2000-12-01  10:50
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 1998



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/


#ifndef GEO57_H
#define GEO57_H



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#if defined(_WIN32)
# include "gos00.h" /* nocheck */             /* prototypes (sql57c_malloc, sql57c_free) only */
# if defined (_DEBUG)
#  include <crtdbg.h>
# endif
#else
#include "gen57.h" /* nocheck */              /* prototypes should be relocated ... see above */
#endif

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/


/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/


/*! Chapter: Macros */

/*!---------------------------------------------------------------------------
  Declaration:  Memory allocation macros
  Description:  ALLOC_MEM, FREE_MEM, ALLOCA ...
*/
#if defined (_WIN32)

# define  ALLOCA_EO57(_s)                _alloca(_s)

# if defined (KERNEL)
#  define  ALLOC_MEM_EO57(_p,_s)         sql57k_pmalloc(__LINE__,__FILE__,_p,_s)
#  define  FREE_MEM_EO57(_p)             sql57k_pfree(__LINE__,__FILE__,_p)
# else
#  define  ALLOC_MEM_EO57(_p,_s)         sql57c_malloc(__LINE__,__FILE__,_p,_s)
#  define  FREE_MEM_EO57(_p)             sql57c_free(__LINE__,__FILE__,_p)
# endif

#else
# define  ALLOCA_EO57(_s)               alloca(_s)
# define  ALLOC_MEM_EO57(_p,_s)         sql57k_pmalloc(__LINE__,__FILE__,_p,_s)
# define  FREE_MEM_EO57(_p)             sql57k_pfree(__LINE__,__FILE__,_p)
#endif

/*! EndDeclaration: Memory allocation macros*/
/*! EndChapter: Macros */


/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

#if defined (__cplusplus)

/*---------------------------------------------------------------------------*/
/*------------------------ class teo57_MemoryState --------------------------*/
/*---------------------------------------------------------------------------*/

#if defined(_WIN32) && defined (_DEBUG)
  /*!---------------------------------------------------------------------------
    Class:        teo57_MemoryState
    Description:  Memory state snapshots and leak detection.

                  The teo57_MemoryState class provides a functionality to 
                  make memory state snapshots and detect memory leaks.

                  teo57_MemoryState does not have a base class.
  */
  class teo57_MemoryState 
  {
  public:
    /*!---------------------------------------------------------------------------
       Function:     teo57_MemoryState
       Description:  Create a new event memory state object.

       Arguments:    none.
    */
    teo57_MemoryState() { 
      memset(this, 0, sizeof(*this)); };
    
    ~teo57_MemoryState() {};
  
    /*!---------------------------------------------------------------------------
       Function:     eo57_SnapShot
       Description:  Make a snapshot of the current memory state

       Arguments:    none.
       Return value: none
    */
    void            eo57_SnapShot() {   
                      _CrtMemCheckpoint( &State_eo57 ); };

    /*!---------------------------------------------------------------------------
       Function:     eo57_Difference
       Description:  Build the difference between two memory snapshots.

       Arguments:    PrevState [in]  - previous snapshots
                     CurrState [in]  - current snapshots
       Return value: Returns true if there are differences.
    */
    bool            eo57_Difference( const teo57_MemoryState& PrevState,
                                     const teo57_MemoryState& CurrState ) {
                      return _CrtMemDifference( &State_eo57, 
                                                &PrevState.State_eo57, 
                                                &CurrState.State_eo57 ) != 0; };
    /*!---------------------------------------------------------------------------
       Function:     eo57_DumpObjects, eo57_DumpStatistics
       Description:  Dump memory information to the 'debugger output window'.

                     Dump objects created after this memory state was checkpointed 
                     Will dump all objects if this memory state wasn't checkpointed
                     Dump all objects, report about non-objects also List request
                     number in {}

       Arguments:    none.
       Return value: none.

       prototypes: 2
    */
    void            eo57_DumpObjects() const { 
                      _CrtMemDumpAllObjectsSince( &State_eo57 ); };

    void            eo57_DumpStatistics() const { 
                      _CrtMemDumpStatistics( &State_eo57 ); };

  private:
    _CrtMemState  State_eo57;
  };
  /*! EndClass: teo57_MemoryState */
#endif

#endif /* __cplusplus */

/*===========================================================================*
 *  FUNCTION PROTOTYPES                                                      *
 *===========================================================================*/


/*===========================================================================*
 *  EXTERNAL VARIABLES                                                       *
 *===========================================================================*/


/*===========================================================================*
 *  EXAMPES                                                                  *
 *===========================================================================*/

/*!
   Example: Example teo57_MemoryState

   Code:
    // Declare the objects needed
    #ifdef _DEBUG
      teo57_MemoryState FirstMemState;
      teo57_MemoryState NextMemState;
      teo57_MemoryState DiffMemState;
    
      FirstMemState.eo57_SnapShot();
    #endif
    
    
    MyLeakyFunc();
    
    
    #ifdef _DEBUG    
      NextMemState.eo57_SnapShot();
    
      if( DiffMemState.eo57_Difference(FirstMemState, NextMemState) )
      {
        DiffMemState.eo57_DumpStatistics();
        DiffMemState.eo57_DumpObjects();      
      }
    #endif
*/


#endif  /* GEO57_H */

==> sapdb-source-7.4.03.32.tgz: V74_03_32/SAPDB_ORG/sys/src/eo/geo57.h <==
/*!**************************************************************************

  module      : geo57.h

  -------------------------------------------------------------------------

  responsible : RaymondR

  special area: Memory management functions.

  description : Memory management functions.


  last changed: 2000-12-05  9:45
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 1998



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/


#ifndef GEO57_H
#define GEO57_H



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#if defined(_WIN32)
# include <gos00.h> /* nocheck */ /* prototypes (sql57c_malloc, sql57c_free) only */
# if defined (_DEBUG)
#  include <crtdbg.h>
# endif
#else
#include "gen57.h" /* nocheck */ /* prototypes should be relocated ... see above */
#endif

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/


/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/


/*! Chapter: Macros */

/*!---------------------------------------------------------------------------
  Declaration:  Memory allocation macros
  Description:  ALLOC_MEM, FREE_MEM, ALLOCA ...
*/
#if defined (_WIN32)

# define  ALLOCA_EO57(_s)                _alloca(_s)

# if defined (KERNEL)
#  define  ALLOC_MEM_EO57(_p,_s)         sql57k_pmalloc(__LINE__,__FILE__,_p,_s)
#  define  FREE_MEM_EO57(_p)             sql57k_pfree(__LINE__,__FILE__,_p)
# else
#  define  ALLOC_MEM_EO57(_p,_s)         sql57c_malloc(__LINE__,__FILE__,_p,_s)
#  define  FREE_MEM_EO57(_p)             sql57c_free(__LINE__,__FILE__,_p)
# endif

#else
# define  ALLOCA_EO57(_s)               alloca(_s)
# define  ALLOC_MEM_EO57(_p,_s)         sql57k_pmalloc(__LINE__,__FILE__,_p,_s)
# define  FREE_MEM_EO57(_p)             sql57k_pfree(__LINE__,__FILE__,_p)
#endif

/*! EndDeclaration: Memory allocation macros*/
/*! EndChapter: Macros */


/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

#if defined (__cplusplus)

/*---------------------------------------------------------------------------*/
/*------------------------ class teo57_MemoryState --------------------------*/
/*---------------------------------------------------------------------------*/

#if defined(_WIN32) && defined (_DEBUG)
  /*!---------------------------------------------------------------------------
    Class:        teo57_MemoryState
    Description:  Memory state snapshots and leak detection.

                  The teo57_MemoryState class provides a functionality to 
                  make memory state snapshots and detect memory leaks.

                  teo57_MemoryState does not have a base class.
  */
  class teo57_MemoryState 
  {
  public:
    /*!---------------------------------------------------------------------------
       Function:     teo57_MemoryState
       Description:  Create a new event memory state object.

       Arguments:    none.
    */
    teo57_MemoryState() { 
      memset(this, 0, sizeof(*this)); };
    
    ~teo57_MemoryState() {};
  
    /*!---------------------------------------------------------------------------
       Function:     eo57_SnapShot
       Description:  Make a snapshot of the current memory state

       Arguments:    none.
       Return value: none
    */
    void            eo57_SnapShot() {   
                      _CrtMemCheckpoint( &State_eo57 ); };

    /*!---------------------------------------------------------------------------
       Function:     eo57_Difference
       Description:  Build the difference between two memory snapshots.

       Arguments:    PrevState [in]  - previous snapshots
                     CurrState [in]  - current snapshots
       Return value: Returns true if there are differences.
    */
    bool            eo57_Difference( const teo57_MemoryState& PrevState,
                                     const teo57_MemoryState& CurrState ) {
                      return _CrtMemDifference( &State_eo57, 
                                                &PrevState.State_eo57, 
                                                &CurrState.State_eo57 ) != 0; };
    /*!---------------------------------------------------------------------------
       Function:     eo57_DumpObjects, eo57_DumpStatistics
       Description:  Dump memory information to the 'debugger output window'.

                     Dump objects created after this memory state was checkpointed 
                     Will dump all objects if this memory state wasn't checkpointed
                     Dump all objects, report about non-objects also List request
                     number in {}

       Arguments:    none.
       Return value: none.

       prototypes: 2
    */
    void            eo57_DumpObjects() const { 
                      _CrtMemDumpAllObjectsSince( &State_eo57 ); };

    void            eo57_DumpStatistics() const { 
                      _CrtMemDumpStatistics( &State_eo57 ); };

  private:
    _CrtMemState  State_eo57;
  };
  /*! EndClass: teo57_MemoryState */
#endif

#endif /* __cplusplus */

/*===========================================================================*
 *  FUNCTION PROTOTYPES                                                      *
 *===========================================================================*/


/*===========================================================================*
 *  EXTERNAL VARIABLES                                                       *
 *===========================================================================*/


/*===========================================================================*
 *  EXAMPES                                                                  *
 *===========================================================================*/

/*!
   Example: Example teo57_MemoryState

   Code:
    // Declare the objects needed
    #ifdef _DEBUG
      teo57_MemoryState FirstMemState;
      teo57_MemoryState NextMemState;
      teo57_MemoryState DiffMemState;
    
      FirstMemState.eo57_SnapShot();
    #endif
    
    
    MyLeakyFunc();
    
    
    #ifdef _DEBUG    
      NextMemState.eo57_SnapShot();
    
      if( DiffMemState.eo57_Difference(FirstMemState, NextMemState) )
      {
        DiffMemState.eo57_DumpStatistics();
        DiffMemState.eo57_DumpObjects();      
      }
    #endif
*/


#endif  /* GEO57_H */
